CleanMinimap = Rock:NewAddon("CleanMinimap", "LibFuBarPlugin-3.0", "LibRockDB-1.0", "LibRockConsole-1.0", "LibRockEvent-1.0", "LibRockTimer-1.0", "LibRockHook-1.0", "LibRockConfig-1.0")
local CleanMinimap = CleanMinimap
CleanMinimap.title = "CleanMinimap"
local shape = "SQUARE";

CleanMinimap:SetDatabase("CleanMinimapDB")
CleanMinimap:SetDatabaseDefaults("profile", {
	UseSquare = true,
	ShowCoord = true,
	ShowZone = true,
	ShowDura = true,
	ShowTrack = true,
	Scale = 1,
	IconScale = 18,
})

CleanMinimap:SetFuBarOption('hasIcon', true)
CleanMinimap:SetFuBarOption('iconPath', "Interface\\AddOns\\CleanMinimap\\icon")
CleanMinimap:SetFuBarOption('defaultPosition', "RIGHT")
CleanMinimap:SetFuBarOption('configType', "LibRockConfig-1.0")
CleanMinimap:SetFuBarOption('cannotDetachTooltip', true)
CleanMinimap:SetFuBarOption('hideWithoutStandby', true)

function GetMinimapShape()
	return shape;
end

function CleanMinimap:OnTextUpdate()
	if self:IsTextShown() then
		self:SetText("CleanMinimap")
	end
	if not self:IsIconShown() then
		self:HideIcon()
	end
end

function CleanMinimap:OnUpdateFuBarTooltip()
	GameTooltip:AddLine("CleanMinimap v" .. GetAddOnMetadata("CleanMinimap", "Version"));
	GameTooltip:AddLine("|cffB39442<Right Click>|r for options");
	fontfile, fontsize, fontflags = GameFontNormalLarge:GetFont();
	GameTooltipTextLeft1:SetFont(fontfile, 15, "OUTLINE");
	fontfile, fontsize, fontflags = GameFontNormal:GetFont();
	GameTooltipTextLeft2:SetFont(fontfile, 13, "OUTLINE");
	GameTooltipTextLeft2:SetTextColor(1, 1, 1);
end

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile =true, tileSize =4,
	edgeFile = "Interface\\Addons\\CleanMinimap\\Texture\\Plain.tga", edgeSize = 4,
	insets = {left = 4, right = 4, top = 4, bottom = 4},
	}
	
local backdropmail = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile =true, tileSize =4,
	edgeFile = "Interface\\Addons\\CleanMinimap\\Texture\\Plain.tga", edgeSize = 4,
	insets = {left = 0, right = 0, top = 0,bottom = 0},
	}

local CleanMinimapFrame = CreateFrame("Frame", nil, Minimap);
local loc, text, track;

function MessageCoords()
	if (UnitInRaid("player")) then
		SendChatMessage("My coordinate location is " .. text:GetText() .. ".", "RAID");
	elseif (GetNumPartyMembers() > 0) then
		SendChatMessage("My coordinate location is " .. text:GetText() .. ".", "PARTY");
	else
		SendChatMessage("My coordinate location is " .. text:GetText() .. ".", "SAY");
	end
end


function CleanMinimap:GetUseSquare()
	return self.db.profile.UseSquare
end
function CleanMinimap:SetUseSquare()
	self.db.profile.UseSquare = not self.db.profile.UseSquare
	if (self.db.profile.UseSquare) then
		MinimapBorder:Hide();
		shape = "SQUARE";
		Minimap:SetMaskTexture("Interface\\AddOns\\CleanMinimap\\texture\\Mask");
		CleanMinimapFrame:Show();
		loc:SetWidth(60);
		text:SetWidth(60);
		loc:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "BOTTOMLEFT", 3, 3)
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Show()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	else
		MinimapBorder:Show();
		shape = "CIRCULAR";
		Minimap:SetMaskTexture("Textures\\MinimapMask");
		CleanMinimapFrame:Hide();
		loc:SetWidth(CleanMinimapFrame:GetWidth());
		text:SetWidth(CleanMinimapFrame:GetWidth());
		loc:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "BOTTOMLEFT", 0, 9)
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Show();
			MinimapZoneTextButton:Hide()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	end
end
function CleanMinimap:GetShowCoord()
	return self.db.profile.ShowCoord
end
function CleanMinimap:SetShowCoord()
	self.db.profile.ShowCoord = not self.db.profile.ShowCoord
	if (self.db.profile.ShowCoord) then
		text:Show()
	else
		text:Hide()
	end
end
function CleanMinimap:GetShowZone()
	return self.db.profile.ShowZone
end
function CleanMinimap:SetShowZone()
	self.db.profile.ShowZone = not self.db.profile.ShowZone
	if (self.db.profile.ShowZone) then
		MinimapZoneTextButton:Show()
	else
		MinimapZoneTextButton:Hide()
	end
	if (self.db.profile.UseSquare) then
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Show()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	else
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Show();
			MinimapZoneTextButton:Hide()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	end
end
function CleanMinimap:GetShowDura()
	return self.db.profile.ShowDura
end
function CleanMinimap:SetShowDura()
	self.db.profile.ShowDura = not self.db.profile.ShowDura
	if (self.db.profile.ShowZone) then
		DurabilityFrame:Show()
		DurabilityFrame:RegisterEvent("UPDATE_INVENTORY_ALERTS")
	else
		DurabilityFrame:Hide()
		DurabilityFrame:UnregisterEvent("UPDATE_INVENTORY_ALERTS")
	end
end
function CleanMinimap:GetShowTrack()
	return self.db.profile.ShowTrack
end
function CleanMinimap:SetShowTrack()
	self.db.profile.ShowTrack = not self.db.profile.ShowTrack
	if (self.db.profile.ShowTrack) then
		self:TrackShow()
	else
		self:TrackHide()
	end
end
function CleanMinimap:GetScale()
	if not self.db.profile then
		return 1
	end
	return self.db.profile.Scale or 1
end
function CleanMinimap:SetScale(size)
	self.db.profile.Scale = size
	MinimapCluster:SetScale(self.db.profile.Scale);
end
function CleanMinimap:GetIconScale()
	if not self.db.profile then
		return 18
	end
	return self.db.profile.IconScale or 18
end
function CleanMinimap:SetIconScale(size)
	self.db.profile.IconScale = size
	MiniMapTracking:SetHeight(self.db.profile.IconScale);
	MiniMapTracking:SetWidth(self.db.profile.IconScale);
	MiniMapTrackingIcon:SetHeight(self.db.profile.IconScale);
	MiniMapTrackingIcon:SetWidth(self.db.profile.IconScale);
	MiniMapMailFrame:SetHeight(self.db.profile.IconScale);
	MiniMapMailFrame:SetWidth(self.db.profile.IconScale);
	MiniMapMailIcon:SetHeight(self.db.profile.IconScale);
	MiniMapMailIcon:SetWidth(self.db.profile.IconScale);
	MiniMapBattlefieldFrame:SetHeight(self.db.profile.IconScale + 15);
	MiniMapBattlefieldFrame:SetWidth(self.db.profile.IconScale + 15);
	MiniMapBattlefieldIcon:SetHeight(self.db.profile.IconScale + 14);
	MiniMapBattlefieldIcon:SetWidth(self.db.profile.IconScale + 14);
end


function CleanMinimap:OnInitialize()
	local options = {
		name = "CleanMinimap",
		desc = self.notes,
		type = 'group',
		args = {
			UseSquare = {
				type = 'toggle',
				name = "Use square minimap",
				desc = "Use the default square minimap.",
				get = "GetUseSquare",
				set = "SetUseSquare",
				order = 1,
			},
			ShowCoord = {
				type = 'toggle',
				name = "Show coordinates",
				desc = "Toggle the coordinates on the minimap.",
				get = "GetShowCoord",
				set = "SetShowCoord",
				order = 2,
			},
			ShowZone = {
				type = 'toggle',
				name = "Show zone header",
				desc = "Toggle the zone header on the minimap.",
				get = "GetShowZone",
				set = "SetShowZone",
				order = 3,
			},
			ShowDura = {
				type = 'toggle',
				name = "Show durability figure",
				desc = "Toggle the durability figure below the minimap.",
				get = "GetShowDura",
				set = "SetShowDura",
				order = 4,
			},
			ShowTrack = {
				type = 'toggle',
				name = "Show tracking icon",
				desc = "Toggle the tracking icon for herbs, minerals, and others.",
				get = "GetShowTrack",
				set = "SetShowTrack",
				order = 5,
			},
			Scale = {
				type = 'number',
				name = "Minimap scale",
				desc = "Increases or decreases the size of the minimap.",
				max = 3,
				min = 0.1,
				step = 0.1,
				get = "GetScale",
				set = "SetScale",
				order = 6,
			},
			IconScale = {
				type = 'number',
				name = "Icon size",
				desc = "Increases or decreases the size of the default icons around the minimap.",
				max = 48,
				min = 6,
				step = 1,
				get = "GetIconScale",
				set = "SetIconScale",
				order = 7,
			},
		}
	}
	self:SetConfigTable(options)

--Restores the saved position of Minimap itself
	local x, y = self.db.profile.posx, self.db.profile.posy 
	if x and y then 
   		local scale = MinimapCluster:GetEffectiveScale() 
		MinimapCluster:ClearAllPoints() 
		MinimapCluster:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / scale, y / scale) 
    end
	
	--self:SetScript("OnEvent",self.OnEvent)
	CleanMinimapFrame:EnableMouseWheel(true)
	CleanMinimapFrame:SetScript("OnMouseWheel", function() self:Zoom() end)
	
	--Setting Up
	CleanMinimapFrame:ClearAllPoints()
	CleanMinimapFrame:EnableMouse(false)
	CleanMinimapFrame:SetPoint("TOPLEFT", Minimap, "TOPLEFT",-3,3)
	CleanMinimapFrame:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT",3,-3)
	CleanMinimapFrame:SetBackdrop(backdrop)
	CleanMinimapFrame:SetBackdropColor(0,0,0,0)
	CleanMinimapFrame:SetBackdropBorderColor(0,0,0,0.6)

	--Coords button
	loc = CreateFrame("Button", nil, Minimap)
	loc:SetWidth(60)
	loc:SetHeight(16)
	loc:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "BOTTOMLEFT", 3, 3)
	loc:SetScript("OnUpdate", function() CleanMinimap:UpdateCoords() end)
	loc:SetScript("OnClick", MessageCoords)

	text = loc:CreateFontString(nil, "OVERLAY","GameFontNormal")
	text:SetWidth(60)
	text:SetHeight(16)
	text:SetPoint("TOP", loc)
	text:SetJustifyH("CENTER")

	--Track Information
	track = CreateFrame("Button", nil, Minimap)
	track:SetWidth(60)
	track:SetHeight(16)
	track:SetPoint("BOTTOMRIGHT", CleanMinimapFrame, "BOTTOMRIGHT", -3, 3)
	track:RegisterForClicks("RightButtonUp")
	track:SetScript("OnClick", function() CancelTrackingBuff() end)
	
	trackt =track:CreateFontString(nil, "OVERLAY","GameFontNormal")
	trackt:SetWidth(60)
	trackt:SetHeight(16)
	trackt:SetPoint("CENTER",track)
	trackt:SetJustifyH("CENTER")
	-- Move zone button
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneText:ClearAllPoints()
	MinimapZoneText:SetPoint("CENTER", MinimapZoneTextButton,"CENTER",0,1)
	MinimapZoneTextButton:SetHeight(21)
	MinimapZoneTextButton:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "TOPLEFT", 0, 2)
	MinimapZoneTextButton:SetPoint("BOTTOMRIGHT", CleanMinimapFrame, "TOPRIGHT", 0, 2)
	MinimapZoneTextButton:SetBackdrop(backdrop)
	MinimapZoneTextButton:SetBackdropColor(0,0,0,0.2)
	MinimapZoneTextButton:SetBackdropBorderColor(0,0,0,0.6)
	MinimapZoneTextButton:SetScript("OnClick", ToggleMinimap)
	 
-- Make map movable on alt 
    MinimapCluster:SetMovable(true) 
    Minimap:SetScript("OnMouseDown", function() 
        if(IsAltKeyDown()) then 
            MinimapCluster:ClearAllPoints() 
			MinimapCluster:StartMoving() 
        else 
            Minimap_OnClick() 
        end 
    end)
    Minimap:SetScript("OnMouseUp", function() 
					MinimapCluster:StopMovingOrSizing()  
					local scale = MinimapCluster:GetEffectiveScale() 
					self.db.profile.posx = MinimapCluster:GetLeft() * scale 
					self.db.profile.posy = MinimapCluster:GetTop() * scale 
					end) 
	 	 
--Minimap extras clean up.
	MinimapZoomIn:Hide();
	MinimapZoomOut:Hide();
	MinimapToggleButton:Hide();
	MiniMapWorldMapButton:Hide();
	GameTimeFrame:Hide();
	MiniMapTracking:SetHeight(self.db.profile.IconScale);
	MiniMapTracking:SetWidth(self.db.profile.IconScale);
	MiniMapTrackingIcon:SetHeight(self.db.profile.IconScale);
	MiniMapTrackingIcon:SetWidth(self.db.profile.IconScale);
	MiniMapTracking:ClearAllPoints();
	MiniMapTracking:SetPoint("TOPRIGHT", Minimap, "TOPLEFT", -6, 4);
	MiniMapTrackingBackground:Hide();
	MiniMapTrackingBorder:Hide();
	
	MiniMapMailFrame:SetHeight(self.db.profile.IconScale);
	MiniMapMailFrame:SetWidth(self.db.profile.IconScale);
	MiniMapMailIcon:SetHeight(self.db.profile.IconScale);
	MiniMapMailIcon:SetWidth(self.db.profile.IconScale);
	MiniMapMailFrame:ClearAllPoints();
	MiniMapMailFrame:SetPoint("TOPLEFT", MiniMapTracking, "BOTTOMLEFT", -5, 0);
	MiniMapMailBorder:Hide();
	
	MiniMapBattlefieldFrame:SetHeight(self.db.profile.IconScale + 15);
	MiniMapBattlefieldFrame:SetWidth(self.db.profile.IconScale + 15);
	MiniMapBattlefieldIcon:SetHeight(self.db.profile.IconScale + 14);
	MiniMapBattlefieldIcon:SetWidth(self.db.profile.IconScale + 14);
	
	MiniMapBattlefieldFrame:ClearAllPoints();
	MiniMapBattlefieldFrame:SetPoint("TOPLEFT", MiniMapMailFrame, "BOTTOMLEFT", 1, -3);
	MiniMapBattlefieldBorder:Hide();

	if (self.db.profile.UseSquare) then
		MinimapBorder:Hide();
		shape = "SQUARE";
		Minimap:SetMaskTexture("Interface\\AddOns\\CleanMinimap\\texture\\Mask");
		CleanMinimapFrame:Show();
		loc:SetWidth(60);
		text:SetWidth(60);
		loc:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "BOTTOMLEFT", 3, 3)
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Show()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	else
		MinimapBorder:Show();
		shape = "CIRCULAR";
		Minimap:SetMaskTexture("Textures\\MinimapMask");
		CleanMinimapFrame:Hide();
		loc:SetWidth(CleanMinimapFrame:GetWidth());
		text:SetWidth(CleanMinimapFrame:GetWidth());
		loc:SetPoint("BOTTOMLEFT", CleanMinimapFrame, "BOTTOMLEFT", 0, 9)
		if (self.db.profile.ShowZone) then
			MinimapBorderTop:Show();
			MinimapZoneTextButton:Hide()
		else
			MinimapBorderTop:Hide();
			MinimapZoneTextButton:Hide()
		end
	end

	if (self.db.profile.ShowCoord) then
		text:Show()
	else
		text:Hide()
	end
	if (self.db.profile.ShowDura) then
		DurabilityFrame:Show()
		DurabilityFrame:RegisterEvent("UPDATE_INVENTORY_ALERTS")
	else
		DurabilityFrame:Hide()
		DurabilityFrame:UnregisterEvent("UPDATE_INVENTORY_ALERTS")
	end
	if (self.db.profile.ShowTrack) then
		self:TrackShow()
	else
		self:TrackHide()
	end
	MinimapCluster:SetScale(self.db.profile.Scale);

--Square Time
	
end

function CleanMinimap:OnEnable()
	self:AddEventListener("PLAYER_LOGIN")
	self:AddEventListener("ZONE_CHANGED_NEW_AREA")
end

function CleanMinimap:OnDisable()
	self:RemoveEventListener("PLAYER_LOGIN")
	self:RemoveEventListener("ZONE_CHANGED_NEW_AREA")
end

--Event Handler
function CleanMinimap:PLAYER_LOGIN()
	SetMapToCurrentZone()
end

function CleanMinimap:ZONE_CHANGED_NEW_AREA()
	SetMapToCurrentZone()
end

--Track things
function CleanMinimap:TrackShow()
	local _, class = UnitClass("player") 
	local t = { 
		["Interface\\Icons\\Ability_Tracking"] = class == "DRUID" and "Humanoids" or "Beasts", 
		["Interface\\Icons\\INV_Misc_Flower_02"] = "Herbs", 
		["Interface\\Icons\\Spell_Nature_Earthquake"] = "Minerals", 
		["Interface\\Icons\\Racial_Dwarf_FindTreasure"] = "Treasure", 
		["Interface\\Icons\\Ability_Tracking"] = "Beasts", 
		["Interface\\Icons\\Spell_Holy_PrayerOfHealing"] = "Humanoids", 
		["Interface\\Icons\\Ability_Stealth"] = "Hidden", 
		["Interface\\Icons\\Spell_Frost_SummonWaterElemental"] = "Elementals", 
		["Interface\\Icons\\Spell_Shadow_DarkSummoning"] = "Undead", 
		["Interface\\Icons\\Spell_Shadow_SummonFelHunter"] = "Demons", 
		["Interface\\Icons\\Ability_Racial_Avatar"] = "Giants", 
		["Interface\\Icons\\INV_Misc_Head_Dragon_01"] = "Dragonkin", 
		["Interface\\Icons\\Spell_Holy_SenseUndead"] = "Undead", 
		["Interface\\Icons\\Spell_Shadow_Metamorphosis"] = "Demons", 
		[""] = "|cffff2020Nothing|r", 
	} 
	--MiniMapTrackingFrame:Hide() 
	--MiniMapTrackingFrame:SetScript("OnEvent", function() trackt:SetText(t[GetTrackingTexture() or ""]) end)
end
	
function CleanMinimap:TrackHide()
	track:Hide()
end

--Slash Handler 
function CleanMinimap:SlashCommandHandler() 
	DEFAULT_CHAT_FRAME:AddMessage("Hold Alt while dragging the map to move the map.\nClick on the coordinates to broadcast your location to party members." .. "\n" .. "Usage: /cmm option" .. "\n" .. "--Options--" .. "\n" .. "|cfffffc00zone|r (Toggle Zone Header)" .. "\n" .. "|cfffffc00track|r (Toggle Tracking Button)" .. "\n" .. "|cfffffc00scale|r {Number from 0.1 to 2} (Sets Map Scaling)" .. "\n" .. "|cfffffc00dur|r (Toggle Durability Frame)" .. "\n" .. "|cfffffc00coord|r (Toggle Coordinates)")
end

--Mousewheel zoom
function CleanMinimap:Zoom()
    if not arg1 then return end
    if arg1 > 0 and Minimap:GetZoom() < 5 then
        Minimap:SetZoom(Minimap:GetZoom() + 1)
    elseif arg1 < 0 and Minimap:GetZoom() > 0 then
        Minimap:SetZoom(Minimap:GetZoom() - 1)
    end
end

--Coord update
function CleanMinimap:UpdateCoords()
    local x, y = GetPlayerMapPosition("player")
    text:SetText(string.format("%.1f,%.1f", 100*x, 100*y))
	--Minimap:SetMaskTexture("Interface\\AddOns\\CleanMinimap\\texture\\Mask");
end
	